// Copyright (C) 2003 Mooffie <mooffie@typo.co.il>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

#include <config.h>

#include "widget.h"
#include "mk_wcwidth.h"
#include "bidi.h"
#include "shaping.h"
#include "my_wctob.h"

Widget::Widget()
{
    wnd = NULL;
    modal = false;
}

Widget::~Widget()
{
    if (wnd)
	delwin(wnd);
}

void Widget::resize(int lines, int columns, int y, int x)
{
    wresize(wnd, lines, columns);
    mvwin(wnd, y, x);
}

// unicode_to_locale() - converts a unicode string to a 8-bit string encoded
// for the current [non-UTF-8] locale.

static void unicode_to_locale(u8string &str, unichar *src, int len)
{
    str.clear();
    while (len--) {
	int ich;
	ich = WCTOB(*src);
	if (ich == EOF)
	    ich = '?';
	str += (char)ich;
	src++;
    }
}

// draw_string() - draws a UTF-8 string. This is a very simple routine and
// it's used by the most simple widgets only (like Label).

void Widget::draw_string(const char *u8, bool align_right)
{
    unistring text, vis_text;
    text.init_from_utf8(u8);

    // trim string to fit window width
    int swidth = 0;
    unichar *trim_pos = text.begin();
    while (trim_pos < text.end()) {
	int char_width = mk_wcwidth(*trim_pos);
	if (swidth + char_width > window_width())
	    break;
	swidth += char_width;
	trim_pos++;
    }
    text.erase(trim_pos, text.end());
   
    // convert to visual
    direction_t dir = BiDi::determine_base_dir(text.begin(), text.size(),
					       algoUnicode);
    BiDi::simple_log2vis(text, dir, vis_text);

    if (terminal::do_arabic_shaping) {
	int new_len = shape(vis_text.begin(), vis_text.len());
	swidth -= vis_text.len() - new_len;
	vis_text.resize(new_len);
    }

    // draw the string
    if (align_right && dir == dirRTL)
	wmove(wnd, 0, window_width() - swidth);
    if (terminal::is_utf8) {
#ifdef HAVE_WIDE_CURSES
	waddnwstr(wnd, (wchar_t*)vis_text.begin(), vis_text.size());
#endif
    } else {
	// 8-bit locale
	u8string local_vis_text;
	unicode_to_locale(local_vis_text, vis_text.begin(), vis_text.size());
	waddnstr(wnd, local_vis_text.c_str(), local_vis_text.size());
    }

    // reposition the cursor
    if (align_right && dir == dirRTL)
	wmove(wnd, 0, window_width() - swidth - 1);
}

void Widget::signal_error()
{
    // I don't like those awful beeps,
    // but people may not be familiar with flashes.

    //flash();
    beep();
}

